package com.sonyericsson.smartphone.graphics;
import java.awt.*;
import sync.Sync;

/**
 * Navigate through a bunch of photos on the screen.
 * The user can tap the screen at upper or lower part
 * to go up down in the list of photos. In this version
 * the photos are loaded from a zip file 'photos.zip'.
 * 
 * @author SEM/GTK/SF Gunnar Forsgren
 * @version 1.0
 
 * some modifications by Metrowerks, Jim Trudeau so that
 * the bean would work inside the QFrame model of panels
 * - created overrides of getPreferredSize and getBounds
 * - moved the key adapter into this class from the app
 * - added commands to go to first and last photos
 * - taps at very top go to first, at bottom go to last
 * - added getCounter and setCounter accessors
 * - added a variety of comments to explain the code
**/


public final class PhotoViewer extends java.awt.Component implements Runnable
{
  java.util.Vector photos=new java.util.Vector();
  Toolkit tk=this.getToolkit().getDefaultToolkit();
  byte[] b;
  sync.Sync client=null;	// reference to user app
  Image newImage=null;		// will hold image as it's created
  int images=0;						// number of photos available
  int counter=0;					// current photo index, 0-based index
 
 	Rectangle bounds = new Rectangle(0,0,208,256); // bounds of the bean
 
  public PhotoViewer()
  {
		setBackground(java.awt.Color.black);
		setSize(208,256);

		// set mouse listener
		SymMouse aSymMouse = new SymMouse();
		this.addMouseListener(aSymMouse);
		
		// set key listener
		SymKey aSymKey = new SymKey();
		this.addKeyListener(aSymKey);
  }


/* Thread that loads photos from zip file */
  public void run()
  {
    loadPhotosFromZip();	
  }


// provide preferred size and bounds
	public Dimension getPreferredSize()
	{
		return new Dimension(208,256);
	}
	
	public Rectangle getBounds()
	{	
		setBounds(0,0,208,256);  // no idea why, component being moved
			
		return bounds;
	}


/* Load photos from zip file 'photos.zip' into a
   'java.util.Vector' object.
   
   For each image loaded, make sure it becomes
   fully loaded before making it available to 
   paint(). For each image loaded it is tracked
   until fully loaded and then added to the
   vector being navigated. This means that more
   images will become available to the user 
   while the loading proceeds. */

  public void loadPhotosFromZip()
  {
    try 
    {
	    MediaTracker mt=new MediaTracker(this);

		// Open the photo file. For the emulator, the file must be
		// in the directory used by the emulator for files
		
	    java.util.zip.ZipFile zf=new java.util.zip.ZipFile("photos.zip");
	    
	    // Get number of entries
	    java.util.Enumeration n=zf.entries();

	    // For each entry in zip file
	    for (int v=0; n.hasMoreElements(); v++)
	    {
	      java.util.zip.ZipEntry ze = (java.util.zip.ZipEntry)n.nextElement();
	      if (ze.isDirectory()==false) 
	      { // If not a directory entry

	        String s=ze.getName();

	        // If gif or jpg
	        if ((s.endsWith(".gif"))||(s.endsWith(".jpg"))) 
	        {
						// set up stream to get the image
						java.io.DataInputStream bi=new java.io.DataInputStream(zf.getInputStream(ze));
						int len=bi.available();
						b=new byte[len];
						bi.readFully(b,0,len);  // Read image into byte array
						bi.close();

						newImage=tk.createImage(b);   // Create image from byte array using Toolkit
						mt.addImage(newImage,images); // Add to image tracker

						try 
						{
						  mt.waitForID(images);  // Wait for image to actually load
						}

						catch (InterruptedException ie)
						{
						}

						photos.addElement(newImage); // Add loaded image to vector
						images++;

						if (images==counter+1) 
						{ // If image matches the last one viewed

						  if (client!=null)
						  {
						    client.ready(); // Signal image loaded
						  }
						  
						  synchronized(this) 
						  { // Prevent thread interference
						    paint(this.getGraphics()); // Paint first image
						  } // end synchronized
						  
						} // if (images == counter + 1)
  	      } // if gif or jpg
				} // If not a directory entry 
	    }  // For each entry in zip file
		} // try LoadPhotos from ZIP
	
		catch (java.io.IOException i) 
		{
		}
		
	} // end function


// Connect viewer to app that uses it
// Lets us notify app that photos are ready
	public void register(Sync client) 
	{
	    this.client=client;
	}


	public void update(Graphics g) 
	{
	    paint(null);
	}


// If an image exists, draw currently selected image
	public void paint(Graphics g) 
	{
	  if (g!=null)
	  {
	    if (images>0)
	    {
				g.drawImage((java.awt.Image)photos.elementAt(counter),0,0,this);
			}
		}
	}


// Navigate among images when upper or lower part of image is pressed.
	public void previous() 
	{
	   counter--;
	   if (counter<0)
	    counter=images-1;
	   repaint();   
	}

	public void next() 
	{
	   counter++;
	   if (counter>=images)
	    counter = 0;
	   repaint();
	}

	public void first() 
	{
		counter = 0;
		repaint();
	}

	public void last() 
	{
		counter = images-1;
		repaint();
	}


// MouseAdapter to handle taps
	class SymMouse extends java.awt.event.MouseAdapter
	{
		public void mousePressed(java.awt.event.MouseEvent event)
		{			
			Object object = event.getSource(); // where did the tap come from
			if (object == PhotoViewer.this)		 // if it is us
			{
				PhotoViewer_MousePressed(event); // process the tap
			}
		}
	}


	public int getCounter()
	{
		return counter;
	}


	public void setCounter(int value)
	{
		counter = value;
	}


// Handle tap in the viewer
//   near the top, go to first, near the bottom, go to last
//   above or below midline, go to previous or next
	 	
	void PhotoViewer_MousePressed(java.awt.event.MouseEvent event)
	{
	    int y=event.getY(); // Screen Y coordinate position tapped
	    
	    if (y<20)	// near the very top
	    {
	    	first();  // go to the first image
	    }
	    
	    else if (y<123)  // if in the upper half
	    {        
	      previous();  // go to the previous image
	    }

	    else if (y>236)  // else if at very bottom
    	{
        last();	// go to the last image
    	}
    
	    else   // else
	    {
	    	next();  // go to next image
	    }
	}
	
		
// KeyAdapter to handle jog dial (and arrow keys)
	class SymKey extends java.awt.event.KeyAdapter
	{
		public void keyPressed(java.awt.event.KeyEvent event)
		{
			int code=event.getKeyCode();

			if (code==38)  // jogdial up (also up arrow on keyboard)
			{
				previous();
			}
			
			else if (code==40)  // jogdial down (down arrow on keyboard)
			{
			  next();
			}
			  
			else if (code==10)  // jogdial select (enter key on keyboard)
			{
				first();
			}		
		}
	} // end SymKey class


} // end viewer class