/**
*Copyright (c) 2001 Symbian Ltd
*/

package com.symbian.devnet.quartz.awt;

import java.awt.*;
import java.awt.event.*;

/** 
* <p>AboutDialog extends java.awt.Dialog to provide an app information screen 
* with a Quartz look & feel.</p>
* <h4> Layout </h4>
* <p>Centres the text horizontally and distributes the lines evenly across 
* the available space	with horizontal and vertical insets of 15.
* If there are more than 6 lines, each line has less than 16 points 
* vertical space which makes the text look cramped, so extend the dialog
* height up to allow each line exactly 16 points. If the total dialog 
* height exceeds 182, the dialog interferes with the menu bar so
* beyond that height there is no alternative but to let the lines be 
* squashed together.</p>
* @author Symbian Devnet
*/

public class AboutDialog extends Dialog implements ActionListener
{
	// canvas dimensions
	private static final int WIDTH = 
		Toolkit.getDefaultToolkit().getScreenSize().width - 10; // offset of 5 on each side 
	private static final int DEFAULT_HEIGHT = 100; // default canvas height
	private static final int MAX_HEIGHT = 182;     // max canvas height
	private static final int MIN_LINE_HEIGHT = 16; // lower limit for text spacing
	// PREF_NUMBER is the maximum number of lines in the dialog before the height is 
	// forced to increase above DEFAULT_HEIGHT.	
	private static final int PREF_NUMBER = DEFAULT_HEIGHT / MIN_LINE_HEIGHT;
	
	/** A button to allow the AboutDialog to be dismissed, i.e. made invisible */
	private Button continueBtn;
	
	/** An array containing the lines of text which will be displayed in the AboutDialog. */
	private  String[] descriptionTexts; 

	/**
	* Creates a modal AboutDialog.
	* @param parent the parent QFrame of this Dialog.
	* @param title title of the AboutDialog.
	* @param _descriptionTexts the description displayed in the dialog box.
	*/
	AboutDialog(QFrame parent, String title, String[] _descriptionTexts)
	{
		super(parent, title);
		setBackground(SystemColor.control);
		descriptionTexts = _descriptionTexts;
		int descLength = descriptionTexts.length;
		
		// Change the canvas height depending on the number of lines in the description text.
		// This ensures that the Dialog will resize as needed.
		int canvasHeight;
		canvasHeight = (descLength <= PREF_NUMBER) ? DEFAULT_HEIGHT : 
			Math.min(MAX_HEIGHT, MIN_LINE_HEIGHT * descLength);
  
		setLayout(new BorderLayout());
		Canvas canvas = new Canvas()
		{
			public void paint(Graphics g)
			{
				paintText(g);
			}
		};
	
		continueBtn = new Button("Continue");
		continueBtn.addActionListener(this);
		Panel panel = new Panel(new BorderLayout());
		canvas.setSize(WIDTH, canvasHeight);
		panel.add(canvas, BorderLayout.CENTER);
		Panel buttonPanel = new Panel(new FlowLayout(FlowLayout.RIGHT));
		buttonPanel.add(continueBtn);
		panel.add(buttonPanel, BorderLayout.SOUTH);
		add(panel);
		pack();
	}
	
	/** This dismisses the AboutDialog (by setting it to invisible) in response to
   * the "Continue" button being pressed. It can be overridden to listen
   * for additional events, but the overriding function should end with a
   * call to <code>super.actionPerformed(ae)</code>.
	 * @param ae an ActionEvent.
	 */
	public void actionPerformed(ActionEvent ae)
	{
		if(ae.getSource() == continueBtn) 
			setVisible(false);					
	}

	/** 
	* ensures that the AboutDialog description text is appropriately laid 
	* out in the canvas.
	* @param g the graphics context of the Canvas.
	*/
	private void paintText(Graphics g)
	{
		FontMetrics fm = g.getFontMetrics();
		int lineCount = descriptionTexts.length;
		for(int i = 0; i < lineCount; i++)
		{
			String desc = descriptionTexts[i];
			// choose an xOffset which centres the text
			int xOffset = (WIDTH - fm.stringWidth(desc))/2;
			// choose a suitable yOffset for this line 
			int yOffset =  DEFAULT_HEIGHT * (i + 1) /	(lineCount + 1) + 1;
			g.drawString(desc, xOffset, yOffset);

		}
	}
	
}