/** 
* Copyright (c) 2001 Symbian Ltd.
*/

package com.symbian.devnet.quartz.awt;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
* The QFrame class provides Java Applications with a Quartz look & feel. It does the
* following:
* <ul>
* <li>Adds a specified number of views as cards in a CardLayout.</li>
* <li>Adds default menus.</li>
* <li>Optionally adds an AboutDialog.</li>
* <li>Provides a version number.</li>
* <li>Guarantees the correct positioning and sizing ofthe Frame below a toolbar and 
* above the status bar.</li>
* </ul>
* This version of the class is designed to operate with Quartz version 6.1 or higher. 
* It takes account of a number of changes made to the AWT implementation.
* The version of this class which shipped with Quartz 6.0 SDKs should not therefore be 
* used with applications written for Quartz 6.1 or vice versa.
* @author Symbian Devnet.
*/
public class QFrame extends Frame 
{
	private final static String VERSION_NAME = "1.3";
		
	/** Contains the two main Menu components.  
	* {@link #appMenu} and {@link #editMenu}
	*/
	protected MenuBar menuBar;
	
	/**  The Application Menu, provided in line with the Quartz style guide */
	protected Menu appMenu;
	
	/**  The Edit Menu, provided in line with the Quartz style guide */
	protected Menu editMenu;
	
	private int numCards;
	private String appName; 
	private CardLayout cLayout; 
	private String[] cardNames;
	private Panel[] panels;
	private boolean aboutDialogDefined = false;
	
	/** Default - creates only one card. */
	public QFrame()
	{
		this(1);
	}
	
	/**
	* Specifies the number of cards to be created.
	* @param _numCards the number of cards to be created. 
	*/
	public QFrame(int _numCards) 
	{		
		appName = System.getProperty("com.symbian.appName");
		int dotIndex = appName.lastIndexOf('/');
		// Default for com.symbian.appName is package qualified class name with "/" separator
		// so strip off the package qualification if necessary
		if (dotIndex != -1 ) appName = appName.substring(dotIndex + 1);
		
		menuBar = new MenuBar();
		editMenu = new Menu("Edit");
		appMenu = new Menu(appName);
		menuBar.add(appMenu);
		menuBar.add(editMenu);
		setMenuBar(menuBar);
			
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent we)
			{
				shutDown();
			}
			
			public void windowActivated(WindowEvent we)
			{
				handleForeground();
			}
			
			public void windowDeactivated(WindowEvent we)
			{
				handleBackground();
			}

		});
		
		pack();		
	
		numCards = _numCards;
		panels = new Panel[numCards];
		cLayout = new CardLayout();
		setLayout(cLayout); 
		cardNames = new String[numCards];
				
		// initialise the card array and add to QFrame.
		for (int i=0; i < numCards; i++)
		{
			panels[i] = new Panel();
			Panel p = panels[i];
			cardNames[i] = "Card" + i;
			add(p, cardNames[i]);
		}
		
		cLayout.first(this); // default to first card
		
	}
	
	/** 
	* Gets the number of available cards.
	* @return the total number of cards available. 
	*/
	public int getCardCount()
	{
		return numCards;	
	}
	
	/** 
	* Gets the card at the specified location. If the specified index is not within range, 
	* null is returned. If the desired <code>Panel</code> has been reset to a subclass 
	* using {@link #setCardAt(Panel, int) setCardAt} then an explicit cast to the subclass 
	* should be performed.
	* @param index indicates the position of the panel to be returned. 
	* @return the specified card if found, otherwise null.
	*/
	public Panel getCardAt(int index) 
	{
		if (index < numCards && index >= 0)
		{
			return panels[index];
		}
		else
		{ 
			return null;
		}
	}
	
	/**
	* Sets the <code>Panel</code> at the specified index of the panels array to be the
	* specified <code>Panel</code> or subclass thereof. The previous <code>Panel</code> 
	* at that position is discarded. The index must be a value greater than or equal to 0 
	* and less than the current number of cards/panels. If this condition is not met,
	* an exception is thrown.
	* If the card to be set is the current card on display then a subsequent call to 
	* {@link #displayCardAt(int index) displayCardAt} will need to be made to update the view.
	* @param panel the  panel to be set.
	* @index the specified index.
	* @throws ArrayIndexOutOfBoundsException if the index is invalid
	*/ 
	protected void setCardAt(Panel panel, int index)
	{
		if (index < numCards && index >= 0)
		{
			remove(panels[index]);
			add(panel, cardNames[index]);
			panels[index] = panel;
		}
		else
		{
			throw new ArrayIndexOutOfBoundsException();
		}
	}
	
	/**
	* Gets all available cards. 
	* @return all cards.
	*/
	public Panel[] getAllCards() 
	{
		Panel[] tempPanels = new Panel[numCards];
		System.arraycopy(panels, 0, tempPanels, 0, numCards);
		return tempPanels;
	}

	/**
	* Displays the panel at a specified position. The specified position
	* must be within bounds else an exception is thrown.
	* @param index indicates the position of the panel to be displayed. 
	* @throws ArrayIndexOutOfBoundsException if the index is invalid
	*/
	public void displayCardAt(int index)
	{
		cLayout.show(this, cardNames[index]);	
	}
	
	/** Displays the next card. */
	public void displayNextCard()
	{
		cLayout.next(this);		
	}
	
	/** Displays the previous card. */
	public void displayPreviousCard()
	{
		cLayout.previous(this);
	}

	/** Displays the first card. */
	public void displayFirstCard()
	{
		cLayout.first(this);
	}
	
	/** Displays the last card. */
	public void displayLastCard()
	{
		cLayout.last(this);
	}
			
	/** 
	* Creates an {@link AboutDialog AboutDialog} and adds an About MenuItem to the App Menu.
	* A default title of "About <i>appName</i>" is used.
	* This method should be invoked at construction time of any derived class.
	* @param description a line of text to appear in the AboutDialog box.
	*/
	public void createAboutDialog(String description)
	{
		createAboutDialog("", new String[] {description});
	}
	
	/** 
	* Creates an {@link AboutDialog AboutDialog} and adds an About MenuItem to the App Menu.
	* A default title of "About <i>appName</i>" is used.
	* This method should be invoked at construction time of the deriving class.
	* @param description an array of strings each of which represents a line of text to 
	* appear in the AboutDialog box.
	*/
	public void createAboutDialog(String[] description)
	{
		createAboutDialog("",  description);
	}
	
	/** 
	* Creates an {@link AboutDialog AboutDialog} with a specified title and adds an
	* About MenuItem to the App Menu.
	* This method should be invoked at construction time of the deriving class.
	* @param title the title of the AboutDialog box. 
	* @param description a line of text to appear in the AboutDialog box.
	*/
	public void createAboutDialog(String title, String  description)
	{
		createAboutDialog(title, new String[] {description});
	}
			
	/** 
	* Creates an {@link AboutDialog AboutDialog} with a specified title and adds an
	* About MenuItem to the App Menu.
	* This method should be invoked at construction time of the deriving class.
	* @param title the title of the AboutDialog box. 
	* @param description an array of strings each of which represents a line of text to 
	* appear in the AboutDialog box.
	*/
	public void createAboutDialog(String title, String[] description)
	{
		if (!aboutDialogDefined)
		{
			title = (title.equals("")) ? "About " + appName : title;
			final AboutDialog aboutDialog = new AboutDialog(this, title, description);
			MenuItem aboutItem = new MenuItem("About");
			aboutItem.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent ae)
				{
					aboutDialog.setVisible(true);
				}
			});
			
			aboutDialog.addWindowListener(new WindowAdapter()
			{
				public void windowDeactivated(WindowEvent we)
				{
					if(aboutDialog.isVisible())
					{
						aboutDialog.setVisible(false);
					}
				}
			});
		
			appMenu.add(aboutItem);
			aboutDialogDefined = true;
		}
	
	}
	
		
	/**
	* Gets the maximum size of this component which for Quartz is the same as 
	* the preferred size.
	* @return the maximum size of this component. 
	*/
	public final Dimension getMaximumSize()
	{
		return getSize();
	}
	
	/**
	* Gets the version name. This method should be overriden in derived classes to 
	* provide the version number of <i>your</i> application.
	* @return the name of this version of <code>QFrame</code>.
	*/
	public String getVersion()
	{
		return VERSION_NAME;
	}
	
	/**
	* Closes down the application. This method is called in response to a WINDOW_CLOSING 
	* event. It should be overriden by derived classes if they have application
	* state/data they need to save before closing down. A call to super.shutDown() should 
	* then follow. 
	*/
	public void shutDown()
	{
		System.exit(0);
	}
	
	/**
	* Invoked on window activation. This should be overridden by subclasses which wish to 
	* handle the WINDOW_ACTIVATED event.
	*/
	public void handleForeground()
	{}
	
	/**
	* Invoked on window deactivation. This should be overridden by subclasses which wish to 
	* handle the WINDOW_DEACTIVATED event.
	*/
	public void handleBackground()
	{}
	
}